from os.path import expanduser

try:
    # For Python 3.5 and later
    import configparser
except ImportError:
    # Fall back to Python 2
    import ConfigParser as configparser


def write(new_creds, profile):
    aws_creds = configparser.ConfigParser()
    home = expanduser('~')
    creds_file = home + '/.aws/credentials'
    aws_creds.read(creds_file)

    if not aws_creds.has_section(profile):
        aws_creds.add_section(profile)

    c = new_creds['Credentials']
    aws_creds.set(profile, 'aws_access_key_id', c['AccessKeyId'])
    aws_creds.set(profile, 'aws_secret_access_key', c['SecretAccessKey'])
    aws_creds.set(profile, 'aws_session_token', c['SessionToken'])
    aws_creds.set(profile, 'aws_session_expiration', c['Expiration'].isoformat())

    with open(creds_file, 'w') as configfile:
        aws_creds.write(configfile)



import base64
import re
import requests
from bs4 import BeautifulSoup
from adfs_aws_login import conf
import xml.etree.ElementTree as ET

try:
    # For Python 3.5 and later
    from urllib.parse import urlunparse
    from urllib.parse import urlparse
except ImportError:
    from urlparse import urlunparse # noqa: F401
    from urlparse import urlparse   # noqa: F401


def _to_str(data):
    ret = data
    decode_method = getattr(data, "decode", None)
    if callable(decode_method):
        try:
            ret = data.decode()
        except:
            ret = _to_str(base64.b64encode(data))
    return str(ret)

class SamlException(Exception):
     pass

def get_saml_assertion(username, password, conf):
    # Initiate session handler
    session = requests.Session()

    # Programmatically get the SAML assertion
    # Opens the initial IdP url and follows all of the HTTP302 redirects, and
    # gets the resulting login page
    formresponse = session.get(conf.ADFS_LOGIN_URL, verify=True)
    # Capture the idpauthformsubmiturl, which is the final url after all the 302s
    idpauthformsubmiturl = formresponse.url

    # Parse the response and extract all the necessary values
    # in order to build a dictionary of all of the form values the IdP expects
    formsoup = BeautifulSoup(_to_str(formresponse.text),"lxml")
    payload = {}

    for inputtag in formsoup.find_all(re.compile('(INPUT|input)')):
        name = inputtag.get('name','')
        value = inputtag.get('value','')
        if "user" in name.lower():
            #Make an educated guess that this is the right field for the username
            payload[name] = username
        elif "email" in name.lower():
            #Some IdPs also label the username field as 'email'
            payload[name] = username
        elif "pass" in name.lower():
            #Make an educated guess that this is the right field for the password
            payload[name] = password
        else:
            #Simply populate the parameter with the existing value (picks up hidden fields in the login form)
            payload[name] = value

    # Some IdPs don't explicitly set a form action, but if one is set we should
    # build the idpauthformsubmiturl by combining the scheme and hostname 
    # from the entry url with the form action target
    # If the action tag doesn't exist, we just stick with the 
    # idpauthformsubmiturl above
    for inputtag in formsoup.find_all(re.compile('(FORM|form)')):
        action = inputtag.get('action')
        loginid = inputtag.get('id')
        if (action and loginid == "loginForm"):
            parsedurl = urlparse(conf.ADFS_LOGIN_URL)
            idpauthformsubmiturl = parsedurl.scheme + "://" + parsedurl.netloc + action

    # Performs the submission of the IdP login form with the above post data
    response = session.post(
        idpauthformsubmiturl, data=payload, verify=True)

    # Overwrite and delete the credential variables, just for safety
    username = '##############################################'
    password = '##############################################'
    del username
    del password

    # Decode the response and extract the SAML assertion
    soup = BeautifulSoup(_to_str(response.text),"lxml")
    assertion = ''

    # Look for the SAMLResponse attribute of the input tag (determined by
    # analyzing the debug print lines above)
    for inputtag in soup.find_all('input'):
        if(inputtag.get('name') == 'SAMLResponse'):
            assertion = inputtag.get('value')

    # Better error handling is required for production use.
    if (assertion == ''):
        raise SamlException('Response did not contain a valid SAML assertion')
        

    # Parse the returned assertion and extract the authorized roles
    awsroles = []
    root = ET.fromstring(base64.b64decode(assertion))
    for saml2attribute in root.iter('{urn:oasis:names:tc:SAML:2.0:assertion}Attribute'):
        if (saml2attribute.get('Name') == 'https://aws.amazon.com/SAML/Attributes/Role'):
            for saml2attributevalue in saml2attribute.iter('{urn:oasis:names:tc:SAML:2.0:assertion}AttributeValue'):
                awsroles.append(saml2attributevalue.text)

    for awsrole in awsroles:
        chunks = awsrole.split(',')
        if'saml-provider' in chunks[0]:
            newawsrole = chunks[1] + ',' + chunks[0]
            index = awsroles.index(awsrole)
            awsroles.insert(index, newawsrole)
            awsroles.remove(awsrole)

    return assertion, awsroles


import sys
from getpass import getpass
from adfs_aws_login import credentials, saml
from adfs_aws_login.conf import init
from threadlocal_aws import region
from threadlocal_aws.clients import sts

try:
    input = raw_input
except NameError:
    pass

def adfs_aws_login():
    conf = init()
    username = None
    # Get the federated credentials from the user
    if not conf.NO_PROMPT:
        sys.stdout.write("Username [" + conf.DEFAULT_USERNAME + "]: ")
        username = input()
    if not username:
        if conf.DEFAULT_USERNAME:
            username = conf.DEFAULT_USERNAME
        else:
            print("Need to give username")
            sys.exit(1)
    password = getpass()

    try:
        assertion, awsroles = saml.get_saml_assertion(username, password, conf)
    except saml.SamlException as e:
        print(e.message)
        sys.exit(1)

    # Overwrite and delete the credential variables, just for safety
    username = '##############################################'
    password = '##############################################'
    del username
    del password

    if not conf.NO_PROMPT:
        # If I have more than one role, ask the user which one they want,
        # otherwise just proceed
        if len(awsroles) > 1:
            i = 0
            print("Please choose the role you would like to assume:")
            for awsrole in awsroles:
                print('[', i, ']: ', awsrole.split(',')[0])
                i += 1
            sys.stdout.write("Selection: ")
            selectedroleindex = input()

            # Basic sanity check of input
            if int(selectedroleindex) > (len(awsroles) - 1):
                print("You selected an invalid role index, please try again")
                sys.exit(1)

            role_arn = awsroles[int(selectedroleindex)].split(',')[0]
            principal_arn = awsroles[int(selectedroleindex)].split(',')[1]
        else:
            role_arn = awsroles[0].split(',')[0]
            principal_arn = awsroles[0].split(',')[1]
    else:
        for awsrole in awsroles:
            if awsrole.startswith(conf.ROLE_ARN + ","):
                role_arn = conf.ROLE_ARN
                principal_arn = awsrole.split(',')[1]
    
    if not role_arn:
        print("No valid role found in assertions")
        sys.exit(3)
    # Use the assertion to get an AWS STS token using Assume Role with SAML
    token = sts().assume_role_with_saml(RoleArn=role_arn, PrincipalArn=principal_arn,
                                        SAMLAssertion=assertion, DurationSeconds=conf.DURATION)
    credentials.write(token, conf.PROFILE)



from os import getenv
from os.path import expanduser, join
import sys
import argparse
import json
import logging

try:
    # For Python 3.5 and later
    import configparser
except ImportError:
    # Fall back to Python 2
    import ConfigParser as configparser  # noqa: F401


CALLBACK_SERVER_TIMEOUT = 30

def init():
    aws_config = configparser.ConfigParser()
    home = expanduser('~')
    aws_config.read(join(home, '.aws', 'config'))
    try:
        my_conf = Config(_parse_args(), aws_config)
    except ConfigExeption as ex:
        print(ex)
        sys.exit(1)

    return my_conf


def _fail_message(missing_config, profile):
    return "No value for required configuration parameter {} in profile {}.".format(missing_config, profile)


def _parse_args():
    parser = argparse.ArgumentParser(description='AWS ADFS Login')
    parser.add_argument('-p', '--profile', type=str, nargs='?', default=getenv('AWS_PROFILE', 'default'),
                        help='AWS profile name to log in with')
    parser.add_argument('-u', '--user', type=str, nargs='?', default=getenv('ADFS_USER', ''),
                        help='Username to log in with')
    parser.add_argument('-n', '--no-prompt', action="store_true", help="Do not prompt for username and AWS role (need to be set in config in this case)")
    parser.add_argument('-d', '--duration', type=int, help="Duration of the session in hours. Defaults to 1.")
    parser.add_argument('-r', '--role', help="The AWS IAM role ARN to assume")
    return parser.parse_args()


class ConfigExeption(Exception):
    pass

class Config:
    def __init__(self, args, aws_config):
        self.PROFILE = args.profile
        self.DEFAULT_USERNAME = args.user
        self.CONFIG_PROFILE = args.profile
        self.NO_PROMPT = args.no_prompt
        self.ROLE_ARN = args.role
        if args.duration:
            self.DURATION = args.duration * 3600
        else:
            self.DURATION = None
        if self.CONFIG_PROFILE != 'default':
            self.CONFIG_PROFILE = 'profile {}'.format(self.PROFILE)
        if not aws_config.has_section(self.CONFIG_PROFILE):
            raise ConfigExeption("Couldn't find configuration for profile: {}".format(self.PROFILE))

        config_section = aws_config[self.CONFIG_PROFILE]
        self.ADFS_LOGIN_URL = config_section.get('adfs_login_url', None)
        if self.ADFS_LOGIN_URL is None:
            raise ConfigExeption(_fail_message('adfs_login_url', self.PROFILE))
        if not self.ROLE_ARN:
            self.ROLE_ARN = config_section.get('adfs_role_arn', None)
        if not self.DEFAULT_USERNAME:
            self.DEFAULT_USERNAME = config_section.get('adfs_default_username', None)
        if not self.DURATION:
            conf_duration = config_section.get('adfs_session_duration', "")
            if conf_duration.isdigit():
                self.DURATION = int(conf_duration) * 3600
        if not self.DURATION:
            self.DURATION = 3600

    def __str__(self):
        return json.dumps({"adfs_url": self.ADFS_LOGIN_URL, "profile": self.PROFILE,
            "username": self.DEFAULT_USERNAME, "profile": self.PROFILE,
            "conf_profile": self.CONFIG_PROFILE, "no-prompt": self.NO_PROMPT,
            "role-arn": self.ROLE_ARN}, indent=2)


if __name__ == "__main__":
    init()



# coding=utf8
from setuptools import setup
import os

dir_path = os.path.dirname(os.path.realpath(__file__))

with open('README.md') as f:
    long_description = f.read()

setup(name='adfs-aws-login',
      version='0.1.8',
      description='CLI login to AWS using ADFS',
      long_description=long_description,
      long_description_content_type='text/markdown',
      url='http://github.com/NitorCreations/adfs-aws-login',
      download_url='https://github.com/NitorCreations/adfs-aws-login',
      author='Pasi Niemi',
      author_email='pasi.niemi@nitor.com',
      license='Apache 2.0',
      packages=['adfs_aws_login'],
      include_package_data=True,
      scripts=[],
      entry_points={
          'console_scripts': ['adfs-aws-login=adfs_aws_login.cli:adfs_aws_login'],
      },
      setup_requires=['pytest-runner'],
      install_requires=[
          'requests==2.22.0',
          'threadlocal-aws==0.6',
          'beautifulsoup4==4.8.1',
          'lxml'],
      tests_require=[
          'pytest==4.6.5',
          'pytest-mock==1.10.4',
          'pytest-cov==2.7.1',
          'requests-mock==1.6.0',
          'pytest-runner',
          'mock==3.0.5',
          'cryptography==3.2.1'
      ],
      test_suite='tests')
